@@ -23,7 +23,7 @@ SpCol<eT>::SpCol()
 
 
 
-//! construct a column vector with the specified number of n_elem
+//! construct a column vector with the specified number of elements
 template<typename eT>
 inline
 SpCol<eT>::SpCol(const uword in_n_elem)
@@ -346,7 +346,7 @@ SpCol<eT>::shed_row(const uword row_num)
   {
   arma_extra_debug_sigprint();
 
-  arma_debug_check( row_num >= SpMat<eT>::n_rows, "Col::shed_row(): out of bounds");
+  arma_debug_check( row_num >= SpMat<eT>::n_rows, "SpCol::shed_row(): out of bounds");
   
   shed_rows(row_num, row_num);
   }
@@ -364,7 +364,7 @@ SpCol<eT>::shed_rows(const uword in_row1, const uword in_row2)
   arma_debug_check
     (
     (in_row1 > in_row2) || (in_row2 >= SpMat<eT>::n_rows),
-    "Col::shed_rows(): indices out of bounds or incorrectly used"
+    "SpCol::shed_rows(): indices out of bounds or incorrectly used"
     );
   
   const uword diff = (in_row2 - in_row1 + 1);
@@ -532,6 +532,7 @@ SpCol<eT>::end_row(const uword row_num) const
   }
 
 
+
 #ifdef ARMA_EXTRA_SPCOL_MEAT
   #include ARMA_INCFILE_WRAP(ARMA_EXTRA_SPCOL_MEAT)
 #endif
